package models;
import utilities.ModelError;

import com.martiansoftware.jsap.*;


/**
* This class will run any of the four interesting models with any of three learning rules
*   but only for a single player.
* 
* @author Kevin Zollman
*
*/



public class SinglePlayerModel {
        
        public static void main(String[] args) {
                
                try {
                // We're using the JSAP system for argument parsing
                
                JSAP jsap = new JSAP();
                
                // -l learning rule
                FlaggedOption lr = new FlaggedOption("learningrule")
                                                .setShortFlag('l')
                                                .setLongFlag("learningrule")
                                                .setRequired(true)
                                                .setDefault("fp")
                                                .setStringParser(JSAP.STRING_PARSER);
                lr.setHelp("Chooses a learning rule: fp for standard Fictitious Play, sfp for smoothed Fictitious Play, and r for reinforcement");
                jsap.registerParameter(lr);
                
                // -d for which model to use
                FlaggedOption mod = new FlaggedOption("model")
                                                .setShortFlag('d')
                                                .setLongFlag("model")
                                                .setRequired(true)
                                                .setDefault("bg")
                                                .setStringParser(JSAP.STRING_PARSER);
                mod.setHelp("Chooses a mode: bg for BalaGoyal Model, dpl for double payoff learning, spl for single payoff learning, and sm for standard model");
                jsap.registerParameter(mod);
                
                // -n does the runs with no bad state
                Switch switchnb = new Switch("noBadState")
                                        .setShortFlag('n')
                                        .setLongFlag("nobadstate");
                switchnb.setHelp("Only runs the good state");
                jsap.registerParameter(switchnb);
                                
                // -v prints detailed output
                FlaggedOption ver = new FlaggedOption("verbose")
                                                .setShortFlag('v')
                                                .setLongFlag("verbose")
                                                .setRequired(true)
                                                .setDefault("0")
                                                .setStringParser(JSAP.INTEGER_PARSER);
                ver.setHelp("0 only summary data, 1 is data on each run, 2 is data on every generation");
                jsap.registerParameter(ver);
                
                // -r <runs> s the number of runs per job, default 40,000
                FlaggedOption runsoption = new FlaggedOption("runs")
                                                .setShortFlag('r')
                                                .setLongFlag("runs")
                                                .setRequired(true)
                                                .setDefault("40000")
                                                .setStringParser(JSAP.LONG_PARSER);
                runsoption.setHelp("Number of runs per job");
                jsap.registerParameter(runsoption);
                
                //-p A payoff to use
                FlaggedOption pay = new FlaggedOption("payoffs")
                                .setShortFlag('p')
                                .setLongFlag("payoff")
                                .setStringParser(JSAP.INTEGER_PARSER)
                                .setRequired(true)
                                .setAllowMultipleDeclarations(true);
                jsap.registerParameter(pay);
                
                 // -b <max_payoff> is the maximum mean payoff for randomly assigning payoffs,
                //   default is 50
                FlaggedOption bigoption = new FlaggedOption("bigpayoff")
                                        .setShortFlag('b')
                                        .setLongFlag("bigpayoff")
                                        .setRequired(true)
                                        .setDefault("50")
                                        .setStringParser(JSAP.INTEGER_PARSER);
                bigoption.setHelp("Maxmimum mean payoff for randomly assigning payoffs");
                jsap.registerParameter(bigoption);
                
                // -g <generations> is the maximum number of generations until give up
                //                  default 2000
                FlaggedOption generations = new FlaggedOption("generations")
                                                .setShortFlag('g')
                                                .setLongFlag("generations")
                                                .setRequired(true)
                                                .setDefault("2000")
                                                .setStringParser(JSAP.INTEGER_PARSER);
                generations.setHelp("Number of generations per run");
                jsap.registerParameter(generations);
                
                
                // -s <stratnum> gives the number of strategies default 2
                FlaggedOption stratnum = new FlaggedOption("stratnum")
                                                .setShortFlag('s')
                                                .setLongFlag("stratnum")
                                                .setRequired(true)
                                                .setDefault("2")
                                                .setStringParser(JSAP.INTEGER_PARSER);
                stratnum.setHelp("Number of states/strategies");
                jsap.registerParameter(stratnum);
                
                // -i  sets the priors to be biased toward the bad state
                Switch biasedpriors = new Switch("biasedPriors")
                                        .setShortFlag('i')
                                        .setLongFlag("biasedpriors");
                biasedpriors.setHelp("Starts the priors biased toward the bad state");
                jsap.registerParameter(biasedpriors);
                
                // -m <rate> sets tbe mutation rate, default 0.0
                FlaggedOption mutationrate = new FlaggedOption("mutationRate")
                                                .setShortFlag('m')
                                                .setLongFlag("mutation")
                                                .setRequired(true)
                                                .setDefault("0.0")
                                                .setStringParser(JSAP.DOUBLE_PARSER);
                mutationrate.setHelp("Sets the probability that an individual randomly changes her beleifs");
                jsap.registerParameter(mutationrate);
                
                // -c turns of bad state convergence detection
                Switch switchbadconv = new Switch("noConvergenceDetect")
                                        .setShortFlag('c')
                                        .setLongFlag("noconvergencedetect");
                switchbadconv.setHelp("Turns off convergence detection for the bad state");
                jsap.registerParameter(switchbadconv);
                
                // -a gamma for smoothed fictitious play
                FlaggedOption gamma = new FlaggedOption("gamma")
                                                .setShortFlag('a')
                                                .setLongFlag("gamma")
                                                .setRequired(true)
                                                .setDefault("0.05")
                                                .setStringParser(JSAP.DOUBLE_PARSER);
                gamma.setHelp("Sets the gamma for smoothed fictitious play");
                jsap.registerParameter(gamma);
                
                // -w the maximum to use for weights in reinforcement learning
                FlaggedOption maxweight = new FlaggedOption("maxWeight")
                                                .setShortFlag('w')
                                                .setLongFlag("weight")
                                                .setRequired(true)
                                                .setDefault("1")
                                                .setStringParser(JSAP.INTEGER_PARSER);
                maxweight.setHelp("Sets the maximum intial weights for reinforcement learning");
                jsap.registerParameter(maxweight);
                
                // -z maximum alpha/beta
                FlaggedOption maxab = new FlaggedOption("alphabeta")
                                                .setShortFlag('z')
                                                .setLongFlag("alphabeta")
                                                .setRequired(true)
                                                .setDefault("4")
                                                .setStringParser(JSAP.INTEGER_PARSER);
                maxab.setHelp("The maximum value for the initial alpha's and beta's.");
                jsap.registerParameter(maxab);
                
                // Process commandline 
                JSAPResult config = jsap.parse(args);   
                if (!config.success()) {
                        System.err.println();
                        System.err.println("Usage: java "
                                + BgModel.class.getName());
                        System.err.println("                "
                                + jsap.getUsage());
                        System.err.println();
                        System.err.println(jsap.getHelp());
                        System.exit(1);
                }
                
                                
                if(config.getString("model").equals("bg")) {
                        SingleBGM model = new SingleBGM(config);
                        model.run(config);
                }
                if(config.getString("model").equals("spl")) {
                        SingleSPL model = new SingleSPL(config);
                        model.run(config);
                }
                if(config.getString("model").equals("dpl")) {
                        SingleDPL model = new SingleDPL(config);
                        model.run(config);
                }
                if(config.getString("model").equals("sm")) {
                        SingleSM model = new SingleSM(config);
                        model.run(config);
                }
                }
                catch(ModelError e) {
                        System.out.println(e);
                }
                catch(JSAPException e) {
                        System.out.println(e);
                }
                
        }
}
