package utilities;
import java.awt.image.*;
import java.util.*;

public class BeliefAnimation {
        
        static final int[] dcolors = new int[] {0xFF0000, 0x0000FF, 0x00FF00};
        
        Individual indiv;
        AnimatedGifEncoder anim;
        BufferedImage prevImage;
        int size;
        int actions;
        
        public void newFrame() throws ModelError {

                // This first block creates a frame with a vertical line drawn for each payoff
                int action = indiv.getStrat();
                int payoff = indiv.getPayoff();
                if (action < 3) {
                        for(int i=0;i<100;i++) 
                                prevImage.setRGB(payoff, i, dcolors[action]);
                }
                
                HashSet neigh = indiv.getConnected();
                synchronized(neigh) {
                Iterator j = neigh.iterator();
                while(j.hasNext()) {
                        Individual n = (Individual) j.next();
                        
                        int neghaction = n.getStrat();
                        int neghpayoff = n.getPayoff();
                        if (neghaction < 3) {
                                for(int i=0;i<100;i++) 
                                        prevImage.setRGB(neghpayoff, i, dcolors[neghaction]);
                        }
                }
                }

                anim.addFrame(prevImage);
                
                //This second block draws a second frame with the new beliefs.
                //Default is white
                for(int i=0;i<size;i++) {
                        for(int j=0;j<100;j++) 
                                prevImage.setRGB(i, j, 0xFFFFFF);
                }
                for(int i=0;i<actions;i++) {
                        for(int j=0;j<indiv.getBeliefs()[i].length;j++) {
                                prevImage.setRGB(j, (int) 100-((int) (indiv.getBelief(i,j)*100)), dcolors[i]);
                        }
                }
                
                anim.addFrame(prevImage);
        }
        
        public void finish() { anim.finish(); }
        
        public void start(Individual ind, String fn) {
                indiv = ind;
                anim = new AnimatedGifEncoder();
                anim.start(fn);
                anim.setDelay(1000);
                size = 0;
                actions = indiv.getBeliefs().length;
                if(actions > 3)
                        actions = 3;
                for(int i=0;i<actions;i++) {
                        if(indiv.getBeliefs()[i].length > size)
                                size = indiv.getBeliefs()[i].length;
                }
                
                
                //The first frame is the prior beliefs.
                BufferedImage b = new BufferedImage(size+1, 101, BufferedImage.TYPE_INT_RGB);
                //Default is white
                for(int i=0;i<size+1;i++) {
                        for(int j=0;j<101;j++) 
                                b.setRGB(i, j, 0xFFFFFF);
                }
                for(int i=0;i<actions;i++) {
                        for(int j=0;j<indiv.getBeliefs()[i].length;j++) {
                                b.setRGB(j, (int) 100-((int) (indiv.getBelief(i,j)*100)), dcolors[i]);
                        }
                }
                
                anim.addFrame(b);
                prevImage = b;
        }
        
        public BeliefAnimation() {}
}
