package utilities;
import java.util.*;
import java.io.*;

/**
* A class for importing and exporting graph object into Individuals
* @author Kevin Zollman
*/
public class GraphIO {
        
        /**
        * A function to read jobs files.  A jobs file is structured as follows: <br>
        * The job id<br>
        * The job description string<br>
        * A graph adjacency matrix<br>
        * repeat<br>
        * This is read in by this function and a HashSet of Jobs objects are returned.
        * @param fn The file name of the jobs file
        * @return A HashSet of Jobs objects read from the jobs file.
        */
        public static HashSet readJobsFile(String fn) 
                throws ModelError, FileNotFoundException, IOException {
                File input = new File(fn);
                FileReader fr = new FileReader(input);
                BufferedReader in = new BufferedReader(fr);
                
                HashSet list = new HashSet();
                String line;
                String descrip;
                int ji;
                int[][] matrix;
                while((line=in.readLine()) != null) {
                        ji = Integer.parseInt(line);
                        if((line=in.readLine()) == null)
                                throw new ModelError("File malformed");
                        descrip = line;
                        if((line=in.readLine()) == null)
                                throw new ModelError("File malformed");
                        matrix = GraphIO.stringMatrix(line);
                        Jobs j = new Jobs(ji, descrip, matrix);
                        list.add(j);
                }
                return list;
        }
        
        /**
        * A function to turn a string representing an adjacency matrix into an integer matrix.
        * THIS IS NOT A GENERALIZED UTILITY, it expects to get one two dimenstional matrix of 
        * single digit integers.  Anything else will screw it up.
        * @param s The string to be processed
        * @return A two dimensional integer matrix processed from the string
        */
        public static int[][] stringMatrix(String s) throws ModelError{
                s = s.trim();
                String[] entries = s.split("}");
                int[][] matrix = new int[entries.length][entries.length];
                for(int i=0;i<entries.length;i++) {
                        String[] words = entries[i].split(" ");
                        int j=0;
                        int k=0;
                        if(words[0].charAt(0) == ',') {
                                k=1;
                                j=1;
                        }
                        for(;j<entries.length+k;j++) {
                                matrix[i][j-k] = Character.digit( words[j].charAt(words[j].lastIndexOf('{')+1), 10);
                        }
                }
                return matrix;
        }
        
        /**
        * Turns an adjacency matrix into a collection of Individuals.  This function does
        *  not allow self connections regardless of what the adjacency matrix says.  For the
        *  matrix, 0 means no connection greater than 0 means a connection.  No weightings
        *  or anything.  
        * @param m The adjacency matrix
        * @return A collection of individuals who are interconnected according to the matrix
        */
        public static Individual[] adjacencyMatrix(int[][] m) {
                Individual[] inds = new Individual[m.length];
                
                for(int i=0;i<m.length;i++) 
                        inds[i] = new Individual(i);
                for(int i=0;i<m.length;i++) {
                        for(int j=0;j<m[i].length;j++) {
                                if(m[i][j] > 0 && i!=j) 
                                        inds[i].addConnected(inds[j]);
                        }
                }
                return inds;
        }
        
}
